/*
    libodsstream is a library to read and write ODS documents as streams
    Copyright (C) 2013  Olivier Langella <Olivier.Langella@moulon.inra.fr>

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

*/

#include "odsdocreader.h"
#include "odsexception.h"
#include <QDebug>
#include "./saxreader/saxhandlercontentxml.h"
#include <quazip5/quazipfile.h>

OdsDocReader::OdsDocReader(OdsDocHandlerInterface &handler) : _handler(handler)
{
}

OdsDocReader::~OdsDocReader()
{
  if(_p_quaZip != 0)
    {
      if(_p_quaZip->isOpen())
        {
          _p_quaZip->close();
          delete _p_quaZip;
          _p_quaZip = 0;
        }
    }
}


void
OdsDocReader::parse(QFile &odsFile)
{
  qDebug() << __FILE__ << " " << __FUNCTION__ << " " << __LINE__;
  odsFile.open(QIODevice::ReadOnly);
  parse(&odsFile);
  odsFile.close();
}

void
OdsDocReader::parse(QIODevice *p_inputstream)
{
  qDebug() << __FILE__ << " " << __FUNCTION__ << " " << __LINE__;
  _p_quaZip = new QuaZip(p_inputstream);
  _p_quaZip->open(QuaZip::mdUnzip);

  qDebug() << __FILE__ << " " << __FUNCTION__ << " " << __LINE__;
  for(bool more = _p_quaZip->goToFirstFile(); more;
      more      = _p_quaZip->goToNextFile())
    {
      qDebug() << _p_quaZip->getCurrentFileName();

      if(_p_quaZip->getCurrentFileName() == "content.xml")
        {

          SaxHandlerContentXml parser(*this);

          QXmlSimpleReader reader;
          reader.setContentHandler(&parser);
          reader.setErrorHandler(&parser);

          qDebug() << __FILE__ << " " << __FUNCTION__ << " " << __LINE__
                   << " Parsing XML input file '"
                   << _p_quaZip->getCurrentFileName() << "'" << endl;

          QuaZipFile zip_file(_p_quaZip);

          QXmlInputSource xmlInputSource(&zip_file);

          if(reader.parse(xmlInputSource))
            {
            }
          else
            {
              qDebug() << __FILE__ << " " << __FUNCTION__ << " " << __LINE__
                       << " error " << parser.errorString();
              throw OdsException(QObject::tr("error reading ODS input file :\n")
                                   .append(parser.errorString()));
            }
          qDebug() << __FILE__ << " " << __FUNCTION__ << " " << __LINE__
                   << " contentXml : DONE on file '"
                   << _p_quaZip->getCurrentFileName() << "'" << endl;
        }
    }
  if(_p_quaZip != 0)
    {
      if(_p_quaZip->isOpen())
        {
          _p_quaZip->close();
          delete _p_quaZip;
          _p_quaZip = 0;
        }
    }
  this->endDocument();
}

void
OdsDocReader::startInsideLine()
{
  _column_number = 0;
  startLine();
}

void
OdsDocReader::setInsideCell(const OdsCell &cell)
{
  _column_number++;
  qDebug() << __FILE__ << " " << __FUNCTION__ << " " << __LINE__ << " "
           << cell.getOfficeValueType() << " " << cell.toString();
  if(cell.isDate())
    {
      qDebug() << __FILE__ << " " << __FUNCTION__ << " " << __LINE__ << " date "
               << cell.getDateTimeValue().toString(Qt::ISODate) << "  "
               << cell.getDateTimeValue().date().day() << "  "
               << cell.getDateTimeValue().date().month();
    }
  else if(cell.isString())
    {
      qDebug() << __FILE__ << " " << __FUNCTION__ << " " << __LINE__
               << " string " << cell.getStringValue();
    }
  else if(cell.isDouble())
    {
      qDebug() << __FILE__ << " " << __FUNCTION__ << " " << __LINE__
               << " double " << cell.getDoubleValue();
    }
  setCell(cell);
}
